%% Solution for Exercise 5: YALMIP for convex optimization
%    TEMPO Summer School on Numerical Optimal Control and Embedded Optimization 
%    University of Freiburg, July 27 - August 7, 2015 
%    Rien Quirynen, Dimitris Kouzoupis and Moritz Diehl 

%% Part 1: The unconstrained minimization problem

clc;
clear all;
close all;

N = 40;

y = sdpvar(N,1);
z = sdpvar(N,1);

mi = 4/N;
Di = 70;
g0 = 9.81;

Vchain = 0;
for i = 1:N
   Vchain = Vchain + g0*mi*z(i);
   if i < N
      Vchain = Vchain + 1/2*Di*((y(i)-y(i+1))^2 + (z(i)-z(i+1))^2);
   end
end

constr = [  y(1) == -2; ...
            z(1) == 1; ...
            y(N) == 2; ...
            z(N) == 1 ];

% Setting options and solving the problem with quadprog:
options = sdpsettings('solver', 'quadprog','verbose',2);
solvesdp(constr, Vchain, options);

Y = double(y); Z = double(z);

figure;
plot(Y,Z,'--or'); hold on;
plot(-2,1,'xg','MarkerSize',10);
plot(2,1,'xg','MarkerSize',10);
xlabel('y'); ylabel('z');
title('Optimal solution hanging chain (without extra constraints)')

%% Part 2: Piecewise linear ground constraints

constr2 = constr;
for i = 1:N
   constr2 = [constr2; z(i) >= 0.5]; 
   constr2 = [constr2; z(i)-0.1*y(i) >= 0.5]; 
end

% Setting options and solving the problem with quadprog:
options = sdpsettings('solver', 'quadprog','verbose',2);
solvesdp(constr2, Vchain, options);

Y2 = double(y); Z2 = double(z);

figure;
plot(Y2,Z2,'--or'); hold on;
plot(-2,1,'xg','MarkerSize',10);
plot(2,1,'xg','MarkerSize',10);
plot([-2 2],[0.5 0.5],':b');
plot([-2 2],[0.3 0.7],':b');
xlabel('y'); ylabel('z');
title('Optimal solution hanging chain (with linear ground constraints)')

%% Part 3: Convex quadratic ground constraint

constr3 = constr;
for i = 1:N
   constr3 = [constr3; z(i) >= -0.2+0.1*y(i)^2]; 
end

% Setting options and solving the problem with SDPT3:
options = sdpsettings('solver', 'SDPT3','verbose',2);
solvesdp(constr3, Vchain, options);

Y3 = double(y); Z3 = double(z);

x = linspace(-2,2,100);
f = -0.2+0.1*x.^2;

figure;
plot(Y3,Z3,'--or'); hold on;
plot(-2,1,'xg','MarkerSize',10);
plot(2,1,'xg','MarkerSize',10);
plot(x,f,':b');
xlabel('y'); ylabel('z');
title('Optimal solution hanging chain (with convex ground constraint)')

%% Part 4: Nonconvex quadratic ground constraint

constr4 = constr;
for i = 1:N
   constr4 = [constr4; z(i) >= -y(i)^2]; 
end

% Setting options and solving the problem with fmincon:
options = sdpsettings('solver', 'fmincon','verbose',2,'usex0',1);
solvesdp(constr4, Vchain, options);

Y4 = double(y); Z4 = double(z);

x = linspace(-1,1,100);
f = -x.^2;

figure;
plot(Y4,Z4,'--or'); hold on;
plot(-2,1,'xg','MarkerSize',10);
plot(2,1,'xg','MarkerSize',10);
plot(x,f,':b');
xlabel('y'); ylabel('z');
title('Optimal solution hanging chain (with nonconvex ground constraint)')

%% Multiple local minima

figure;
plot(Y4,Z4,'--or'); hold on;
plot(-2,1,'xg','MarkerSize',10);
plot(2,1,'xg','MarkerSize',10);
plot(x,f,':b');
xlabel('y'); ylabel('z');
title('Multiple local minima (with nonconvex ground constraint)')

% Assign all variables on the left side:
assign(y,-2*ones(N,1))
assign(z,1*ones(N,1))

solvesdp(constr4, Vchain, options);

Y5 = double(y); Z5 = double(z);
plot(Y5,Z5,'--og');

% Assign all variables on the right side:
assign(y,2*ones(N,1))
assign(z,1*ones(N,1))

solvesdp(constr4, Vchain, options);

Y6 = double(y); Z6 = double(z);
plot(Y6,Z6,'--ob');
